package ar.edu.utn.frba.tadp.subastas.domain;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import ar.edu.utn.frba.tadp.subastas.exceptions.BusinessException;

public class Subasta {

	private String producto;
	private BigDecimal precioBase;
	private Usuario usuario;
	private Date fechaFin;
	private List<Oferta> ofertas;
	private List<OfertaObserver> ofertaObservers;
	
	public Subasta(String producto, BigDecimal precioBase, Usuario usuario, Date fechaFin) {
		this.producto = producto;
		this.precioBase = precioBase;
		this.usuario = usuario;
		this.fechaFin = fechaFin;
		this.ofertas = new ArrayList<Oferta>();
		this.ofertaObservers = new ArrayList<OfertaObserver>();
	}

	public void ofertar(Usuario usuario, BigDecimal monto) {
		if (this.isFinalizada()) {
			throw new BusinessException("La subasta finaliz a las " + getFechaFinAsString());
		}
		Oferta ofertaActual = new Oferta(usuario, monto);
		if (this.noHayOfertas() || ofertaActual.supera(this.getMejorOferta())) {
			ofertas.add(ofertaActual);
		} // else no estoy haciendo un throw por las ofertas automticas
		System.out.println("Usuario " + usuario + " ofert " + monto);
		for (OfertaObserver ofertaObserver : ofertaObservers) {
			ofertaObserver.notificarNuevaOferta(this);
		}
	}

	public boolean noHayOfertas() {
		return this.ofertas.isEmpty();
	}

	public boolean isFinalizada() {
		return new Date().after(this.fechaFin);
	}

	private String getFechaFinAsString() {
		return new SimpleDateFormat("dd/MM/yyyy hh:mm:ss").format(this.fechaFin);
	}
	
	public Oferta getMejorOferta() {
		Oferta mejorOferta = null;
		for (Oferta oferta : ofertas) {
			if (mejorOferta == null || oferta.supera(mejorOferta)) {
				mejorOferta = oferta;
			}
		}
		return mejorOferta;
	}

	public void agregarOfertaObserver(OfertaObserver ofertaObserver) {
		ofertaObservers.add(ofertaObserver);
	}
	
	public void removeOfertaObserver(OfertaObserver ofertaObserver) {
		ofertaObservers.remove(ofertaObserver);
	}
	
	public void ofertarAutomaticamente(Usuario usuario, BigDecimal ofertaBase, BigDecimal incremento, BigDecimal valorMaximo) {
		this.agregarOfertaObserver(new OfertaAutomatica(usuario, ofertaBase, incremento, valorMaximo));
		this.ofertar(usuario, ofertaBase);
	}
}
