package ar.edu.utn.frba.tadp.subastas.domain;

import java.math.BigDecimal;
import java.util.Date;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import ar.edu.utn.frba.tadp.subastas.exceptions.BusinessException;

public class TestOfertas {

	private Usuario percudani;
	private Usuario olguin;
	private Usuario villaverde;
	private Usuario marangoni;
	private Usuario pavoni;
	private Subasta playstation;
	private Subasta playstationFinalizada;
	
	@Before
	public void setUp() throws Exception {
		percudani = new Usuario("Jose Percudani");
		olguin = new Usuario("Jorge Olgun");
		villaverde = new Usuario("Hugo Villaverde");
		marangoni = new Usuario("Claudio Marangoni");
		pavoni = new Usuario("Ricardo Pavoni");
		playstation = new Subasta("Playstation I", new BigDecimal(200), percudani, new Date(108, 11, 11));
		playstationFinalizada = new Subasta("Playstation I", new BigDecimal(200), percudani, new Date(103, 11, 11));
	}

	@Test (expected=BusinessException.class)
	public void finalizada() {
		playstationFinalizada.ofertar(olguin, new BigDecimal(270));
	}

	@Test
	public void ofertaManual() {
		playstation.ofertar(olguin, new BigDecimal(270));
		Assert.assertEquals(playstation.getMejorOferta().getValor(), new BigDecimal(270));
	}
	
	@Test
	public void ofertaAutomatica() {
		playstation.ofertar(olguin, new BigDecimal(270));
		playstation.ofertarAutomaticamente(villaverde, new BigDecimal(280), new BigDecimal(70), new BigDecimal(550));
		Assert.assertEquals(playstation.getMejorOferta().getValor(), new BigDecimal(280));
	}
	
	@Test
	public void ofertaAutomaticaEntreDos() {
		playstation.ofertar(olguin, new BigDecimal(270));
		playstation.ofertarAutomaticamente(villaverde, new BigDecimal(280), new BigDecimal(70), new BigDecimal(450));
		playstation.ofertarAutomaticamente(pavoni, new BigDecimal(290), new BigDecimal(50), new BigDecimal(550));
		Assert.assertEquals(playstation.getMejorOferta().getValor(), new BigDecimal(410));
	}
	
	@Test
	public void ofertaMix() {
		playstation.ofertar(olguin, new BigDecimal(270));
		playstation.ofertarAutomaticamente(villaverde, new BigDecimal(280), new BigDecimal(70), new BigDecimal(450));
		playstation.ofertarAutomaticamente(pavoni, new BigDecimal(290), new BigDecimal(50), new BigDecimal(550));
		playstation.ofertar(marangoni, new BigDecimal(480));
		Assert.assertEquals(playstation.getMejorOferta().getValor(), new BigDecimal(530));
	}
}
